'The MIT License (MIT)

'Copyright (c) 2010 rdagger

'Permission is hereby granted, free of charge, to any person obtaining a copy
'of this software and associated documentation files (the "Software"), to deal
'in the Software without restriction, including without limitation the rights
'to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
'copies of the Software, and to permit persons to whom the Software is
'furnished to do so, subject to the following conditions:

'The above copyright notice and this permission notice shall be included in all
'copies or substantial portions of the Software.

'THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
'IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
'FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
'AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
'LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
'OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
'SOFTWARE.


'Test setup is on the STK500 with Vtarget set to 3.6V
'Using an atmega644 and external 12MHz crystal

'when you connect the RS232 SPARE to your COM port of your PC
'you must connect TXD SPARE to PORTD.1 (txd pin)

'Save about 38 bytes of code size
$noramclear


'#if Usbdebug = 2
'$dbg
'#endif

$hwstack = 64
$swstack = 64
$framesize = 64                                             '24 bytes reserved


'$regfile = "m644def.dat"
$regfile = "m8def.dat"
'$regfile = "ATtiny461.dat"

$crystal = 12000000



$eepromhex                                                  'for STK500 programmer


'Include the software USB library
$lib "swusb.lbx"
$external _swusb

Declare Sub Usb_reset()
Declare Sub Usb_processsetup(txstate As Byte)
Declare Sub Usb_send(txstate As Byte , Byval Count As Byte)
Declare Sub Usb_senddescriptor(txstate As Byte , Maxlen As Byte)

Declare Sub Display_char(byval Char As Byte)
Declare Sub Push_char(byval Char As Byte)
Declare Sub Set_intensity(byval Level As Byte)
Declare Function Get_rotary() As Byte

Config Adc = Single , Prescaler = Auto , Reference = Avcc   'Initialize ADC






'*******************************************************************************
'*************************** Begin USB Configuration ***************************
'
'Set the following parameters to match your hardware configuration and USB
'device parameters.

'******************************* USB Connections *******************************

'Define the AVR port that the two USB pins are connected to
_usb_port Alias Portd
_usb_pin Alias Pind
_usb_ddr Alias Ddrd

'Define the D+ and D- pins. (put D+ on an interrupt pin)
Const _usb_dplus = 2
Const _usb_dminus = 3

'Configure the pins as inputs
Config Pind.2 = Input
Config Pind.3 = Input

'disable pullups
_usb_port._usb_dplus = 0
_usb_port._usb_dminus = 0

'*******************************************************************************
'************************* USB Configuration Constants *************************

'Use EEPROM or FLASH to store USB descriptors
'1 = EEPROM, 0 = FLASH.  Storing to EEPROM will reduce code size slightly.
Const _usb_use_eeprom = 0

'Don't wait for sent packets to be ACK'd by the host before marking the
'transmission as complete.  This option breaks the USB spec but improves
'throughput with faster polling speeds.
'This may cause reliability issues.  Should leave set to 0 to be safe.
Const _usb_assume_ack = 0

'  *************************** Device Descriptor *****************************

'USB Vendor ID and Product ID (Assigned by USB-IF)
Const _usb_vid = &HAAAA
Const _usb_pid = &HEF02

'USB Device Release Number (BCD)
Const _usb_devrel = &H0001

'USB Release Spec (BCD)
Const _usb_spec = &H0110

'USB Device Class, subclass, and protocol (assigned by USB-IF).
'&h00 = Class defined by interface. (HID is defined in the interface)
'&hFF = Vendor-defined class (You must write your own PC driver)
'See http://www.usb.org/developers/defined_class  for more information
Const _usb_devclass = 0
Const _usb_devsubclass = 0
Const _usb_devprot = 0

'These are _indexes_ to UNICODE string descriptors for the manufacturer,
'product name, and serial number.  0 means there is no descriptor.
Const _usb_imanufacturer = 1
Const _usb_iproduct = 2
Const _usb_iserial = 0

'Number of configurations for this device.  Don't change this unless
'you know what you are doing.  Ordinarily it should just be 1.
Const _usb_numconfigs = 1

'  *************************** Config Descriptor *****************************

'The number of interfaces for this device (Typically 1)
Const _usb_numifaces = 1

'Configuration Number (do not edit)
Const _usb_confignum = 1

'Index of UNICODE string descriptor that describes this config (0 = None)
Const _usb_iconfig = 2

'&H80 = device powered from USB bus.
'&HC0 = self-powered (has a power supply)
Const _usb_powered = &H80

'Required current in 2mA increments (500mA max)
Const _usb_maxpower = 150                                   '150 * 2mA = 300mA

'  ************************** Interface Descriptor ***************************

'Number of interfaces for this device (1 or 2)
Const _usb_ifaces = 1

'Interface number
Const _usb_ifaceaddr = 0

'Alternate index
Const _usb_alternate = 0

'Number of endpoints for this interface (excluding endp 0)
Const _usb_ifaceendpoints = 1                               '**CHANGED from 1 TO 2

'USB Interface Class, subclass, and protocol (assigned by USB-IF).
'&h00 = RESERVED
'&hFF = Vendor-defined class (You must write your own PC driver)
'       Other values are USB interface device class. (such as HID)
'See http://www.usb.org/developers/defined_class  for more information
Const _usb_ifclass = 3
Const _usb_ifsubclass = 0
Const _usb_ifprotocol = 0

'Index to UNICODE string descriptor for this interface (0 = None)
Const _usb_iiface = 0

'  ************************* Optional HID Descriptor *************************
'HID class devices are things like keyboard, mouse, joystick.
'See http://www.usb.org/developers/hidpage/  for the specification,
'tools, and resources.

'Note that for a HID device, the device class, subclass, and protocol
'must be 0. The interface class must be 3 (HID).
'Interface subclass and protocol must be 0 unless you are making a
'keyboard or a mouse that supports the predefined boot protocol.
'See pages 8 and 9 of the HID 1.11 specification PDF.

'Number of HID descriptors (EXCLUDING report and physical)
'If you are not making a HID device, then set this constant to 0
Const _usb_hids = 1

'BCD HID releasenumber.  Current spec is 1.11
Const _usb_hid_release = &H0111

'Country code from page 23 of the HID 1.11 specifications.
'Usually the country code is 0 unless you are making a keyboard.
Const _usb_hid_country = 0

'The number of report and physical descriptors for this HID
'Must be at least 1! All HID devices have at least 1 report descriptor.
Const _usb_hid_numdescriptors = 1

'Use a software tool to create the report descriptor and $INCLUDE it.


'  ************************* Endpoint Descriptor(s) **************************

'Endpoint 0 is not included here.  These are only for optional
'endpoints.
'Note: HID devices require 1 interrupt IN endpoint

'Address of optional endpoints (Must be > 0. comment-out to not use)
Const _usb_endp2addr = 1
'Const _usb_endp3addr = 2                                    '***Could be used for an output

'Valid types are 0 for control or 3 for interrupt
Const _usb_endp2type = 3
Const _usb_endp3type = 0                                    '***Not used

'Directions are: 0=Out, 1=In.  Ignored by control endpoints
Const _usb_endp2direction = 1
Const _usb_endp3direction = 0                               '***Not used

'Polling interval (ms) for interrupt endpoints.  Ignored by control endpoints
' (Must be at least 10)
Const _usb_endp2interval = 10                               '***MAY WANT TO INCREASE BOTH TO 200 AND 100
Const _usb_endp3interval = 10

'*******************************************************************************
'The includes need to go right here--between the configuration constants above
'and the start of the program below.  The includes will automatically calculate
'constants based on the above configuration, dimension required variables, and
'allocate transmit and receive buffers.  Nothing inside the includes file needs
'to be modified.
$include "swusb-includes.bas"
'*******************************************************************************

'**************************** USB Interrupt And Init ***************************
'Set all the variables, flags, and sync bits to their initial states


Call Usb_reset()

Const _usb_intf = Intf0
Config Int0 = Rising
On Int0 Usb_isr Nosave
Enable Int0
Enable Interrupts

'*******************************************************************************
'*************************** End Of USB Configuration **************************




'Set port pins (1 = Output, 0 = Input)
'B: 0,1,2 Max Din, Clk, & Load;  3,4,5 BCD Outputs 0,1,2;
Ddrb = &B00111111
'Set port B output & input states 0 = low/pull-up off, 1 = high/pull-up on
Portb = &B00111000

'C:  0 Pot (feedrate); 1 Extra SW;   2,3,4,5 up,down,left,right
Ddrc = &B00000000
Portc = &B11111110

'D:  0 Pot SW;  1 Mode SW;  2,3 USB+-;  4 eStop SW;  5,6,7 Rotary
Ddrd = &B00000000
Portd = &B11110011

Const F = 70                                                'Declare constants
Const X = 88
Const Y = 89
Const Z = 90
Const Lock = 255

Const X_step_plus = 0
Const X_step_minus = 1
Const Y_step_minus = 2
Const Y_step_plus = 3
Const Estop = 4
Const X_zero = 5
Const Y_zero = 6
Const Z_zero = 7
Const Z_step_minus = 8
Const Z_step_plus = 9
Const Fr_minus = 10
Const Fr_plus = 11
Const Fr_reset = 12
Const Fr_hold = 13
Const Regenerate = 14
Const Cycle_start = 15

Const Center_stick = 15

'Rotation status  constants
Const Rotate_ready = 0
Const Rotating_ccw = 1
Const Rotating_cw = 2
Const Rotate_waiting = 3

'Number of CPU cycles to fire key down event  (1600 good base @ 8 MHz)
'May need to lower
Const Key_down_cycles = 1600


'Number of CPU cycles to pause between key events  (50 good base)
Const Key_wait_cycles = 50

Disp_din Alias Portb.0
Disp_clk Alias Portb.1
Disp_load Alias Portb.2

Bcd0 Alias Portb.3
Bcd1 Alias Portb.4
Bcd2 Alias Portb.5

Zero_switch Alias Pind.0
Axis_switch Alias Pind.1
Estop_switch Alias Pind.4
Rotary1 Alias Pind.5
Rotary2 Alias Pind.6
Rotary3 Alias Pind.7

Regen_switch Alias Pinc.1
Joy_up Alias Pinc.2
Joy_down Alias Pinc.3
Joy_left Alias Pinc.4
Joy_right Alias Pinc.5

Dim Joystick_current As Word                                'Declare variables
Dim Joystick_last As Word
Dim Button_current As Word
Dim Button_last As Word
Dim Resetcounter As Word
Dim Idlemode As Byte

Dim Axis_switch_down As Bit
Dim B As Byte
Dim Bminus As Byte
Dim Bplus As Byte
Dim W As Word
Dim Current_axis As Byte
Dim Disp_addr As Byte
Dim Disp_data As Byte
Dim Spinner_level As Byte
Dim Bcd_level As Byte

'Player rotation status
Dim Rotary_status As Byte
Rotary_status = Rotate_ready

'Rotary joystick current and last postitions
Dim Rotary_current As Byte , Rotary_old As Byte ,

'Rotary joystick difference from last position
Dim Rotary_dif As Integer

'Rotation bi-directional stacks
Dim Rotary_stack As Integer
Rotary_stack = 0

'Counter for rotation status
Dim Rotary_clock As Integer
Rotary_clock = 0

'Set last rotary joystick position to current joystick position
Rotary_old = Get_rotary()


Gosub Max7221_init                                          'Initialize LED matrix display
Gosub Clear_display
Set_intensity &H0F                                          'Set Matrix brightness to high
Display_char X                                              'Set display to X
Current_axis = X




Do
   Resetcounter = 0

   'Check for reset here
   While _usb_pin._usb_dminus = 0

      Incr Resetcounter
      If Resetcounter = 1000 Then
         Call Usb_reset()
      End If
   Wend

   'Check for received data
   If _usb_status._usb_rxc = 1 Then

      If _usb_status._usb_setup = 1 Then
         'Process a setup packet/Control message
         Call Usb_processsetup(_usb_tx_status)
      'else

      End If

      'Reset the RXC bit and set the RTR bit (ready to receive a new packet)
      _usb_status._usb_rtr = 1
      _usb_status._usb_rxc = 0
   End If




   'Lookup spinner axis values
   Bminus = Lookup(spinner_level , Axis_negative)
   Bplus = Lookup(spinner_level , Axis_positive)

   If Bcd_level = 0 Or Current_axis = F Or Current_axis = Lock Then
      'No movement because spinner at 0 or axis not x,y or z
      'Center all axis
      W = Center_stick
      Joystick_current = W
      Shift W , Left , 5
      Joystick_current = Joystick_current Or W
      Shift W , Left , 5
      Joystick_current = Joystick_current Or W
   Else
      'Joystick movement
      Joystick_current = 0
      If Current_axis = X Or Current_axis = Y Then
         'X Axis
         If Joy_left = 0 Then
            Joystick_current = Bminus
         Elseif Joy_right = 0 Then
            Joystick_current = Bplus
         Else
            Joystick_current = Center_stick
         End If
         'Y axis
         If Joy_down = 0 Then
            W = Bminus
            Shift W , Left , 5
            Joystick_current = Joystick_current Or W
         Elseif Joy_up = 0 Then
            W = Bplus
            Shift W , Left , 5
            Joystick_current = Joystick_current Or W
         Else
            W = Center_stick
            Shift W , Left , 5
            Joystick_current = Joystick_current Or W
         End If
         'Only X or Y so center Z
         W = Center_stick
         Shift W , Left , 10
         Joystick_current = Joystick_current Or W
      Elseif Current_axis = Z Then
         'Z Axis
          If Joy_down = 0 Then
            W = Bminus
            Shift W , Left , 10
            Joystick_current = Joystick_current Or W
         Elseif Joy_up = 0 Then
            W = Bplus
            Shift W , Left , 10
            Joystick_current = Joystick_current Or W
         Else
            W = Center_stick
            Shift W , Left , 10
            Joystick_current = Joystick_current Or W
         End If
         'Only Z so center X & Y
         W = Center_stick
         Joystick_current = Joystick_current Or W
         Shift W , Left , 5
         Joystick_current = Joystick_current Or W
      End If
   End If

   'Check for any rotary input
   Gosub Handle_joystick_rotation

   'Set buttons
    Button_current = 0
    Button_current.estop = Not Estop_switch
   'Set other buttons

   If Current_axis = X Then
      'X rotary
      If Rotary_status = Rotating_ccw Then
         Button_current.x_step_minus = 1
      Elseif Rotary_status = Rotating_cw Then
         Button_current.x_step_plus = 1
      End If
      'X zero
      If Zero_switch = 0 Then
         Button_current.x_zero = 1
      End If
      'Regenerate Toolpath
      If Regen_switch = 0 Then
         Button_current.regenerate = 1
      End If
   Elseif Current_axis = Y Then
      'Y rotary
      If Rotary_status = Rotating_ccw Then
         Button_current.y_step_minus = 1
      Elseif Rotary_status = Rotating_cw Then
         Button_current.y_step_plus = 1
      End If
      'Y zero
      If Zero_switch = 0 Then
         Button_current.y_zero = 1
      End If
      'Regenerate Toolpath
      If Regen_switch = 0 Then
         Button_current.regenerate = 1
      End If
   Elseif Current_axis = Z Then
      'Z rotary
      If Rotary_status = Rotating_ccw Then
         Button_current.z_step_minus = 1
      Elseif Rotary_status = Rotating_cw Then
         Button_current.z_step_plus = 1
      End If
      'Z zero
      If Zero_switch = 0 Then
         Button_current.z_zero = 1
      End If
      'Regenerate Toolpath
      If Regen_switch = 0 Then
         Button_current.regenerate = 1
      End If
   Elseif Current_axis = F Then
      'Feedrate rotary
      If Rotary_status = Rotating_ccw Then
         Button_current.fr_minus = 1
      Elseif Rotary_status = Rotating_cw Then
         Button_current.fr_plus = 1
      End If
      'Reset Feedrate
      If Zero_switch = 0 Then
         Button_current.fr_reset = 1
      End If
      'Feedrate Hold
      '****need code
      'Cycle Start
      '****need code
   End If


   If Joystick_current <> Joystick_last Or Button_current <> Button_last Or Idlemode = 0 Then
      'Queue data to be sent on endpoint 2 (HID report)
      If _usb_tx_status2._usb_txc = 1 Then
         Joystick_last = Joystick_current



         _usb_tx_buffer2(2) = Low(joystick_current)         'Joystick data
         _usb_tx_buffer2(3) = High(joystick_current)
         _usb_tx_buffer2(4) = Low(button_current)           'Button data
         _usb_tx_buffer2(5) = High(button_current)



         Call Usb_send(_usb_tx_status2 , 4)
      End If
   End If



   If Axis_switch_down = 1 Then                             'Check if mode switch released
      If Axis_switch = 1 Then
         Axis_switch_down = 0
      End If
   Else
      If Axis_switch = 0 And Current_axis <> Lock Then      'Check if mode switch pressed
         Axis_switch_down = 1

         If Current_axis = Z Then
            Push_char F
         Elseif Current_axis = F Then
            Push_char X
         Else
            B = Current_axis + 1
            Push_char B
         End If
      End If
   End If

   Gosub Poll_feedrate


Loop

End











'*******************************************************************************
'******************** Descriptors stored in EEPROM or FLASH ********************
'                  Do not change the order of the descriptors!
'
#if _usb_use_eeprom = 1
   $eeprom
#else
   $data
#endif

'Device Descriptor
_usb_devicedescriptor:
Data 18 , 18 , _usb_desc_device , _usb_specl , _usb_spech , _usb_devclass
Data _usb_devsubclass , _usb_devprot , 8 , _usb_vidl , _usb_vidh , _usb_pidl
Data _usb_pidh , _usb_devrell , _usb_devrelh , _usb_imanufacturer
Data _usb_iproduct , _usb_iserial , _usb_numconfigs


'Retrieving the configuration descriptor also gets all the interface and
'endpoint descriptors for that configuration.  It is not possible to retrieve
'only an interface or only an endpoint descriptor.  Consequently, this is a
'large transaction of variable size.
_usb_configdescriptor:
Data _usb_descr_total , 9 , _usb_desc_config , _usb_descr_totall
Data _usb_descr_totalh , _usb_numifaces , _usb_confignum , _usb_iconfig
Data _usb_powered , _usb_maxpower

'_usb_IFaceDescriptor
Data 9 , _usb_desc_iface , _usb_ifaceaddr , _usb_alternate
Data _usb_ifaceendpoints , _usb_ifclass , _usb_ifsubclass , _usb_ifprotocol
Data _usb_iiface

#if _usb_hids > 0
'_usb_HIDDescriptor
Data _usb_hid_descr_len , _usb_desc_hid , _usb_hid_releasel , _usb_hid_releaseh
Data _usb_hid_country , _usb_hid_numdescriptors

'Next follows a list of bType and wLength bytes/words for each report and
'physical descriptor.  There must be at least 1 report descriptor.  In practice,
'There are usually 0 physical descriptors and only 1 report descriptor.
Data _usb_desc_report
Data 64 , 0                                                 'Should be 1 less than HID descriptor report length
'End of report/physical descriptor list
#endif

#if _usb_endpoints > 1
'_usb_EndpointDescriptor
Data 7 , _usb_desc_endpoint , _usb_endp2attr , _usb_endp2type , 8 , 0
Data _usb_endp2interval
#endif

#if _usb_endpoints > 2
'_usb_EndpointDescriptor
Data 7 , _usb_desc_endpoint , _usb_endp3attr , _usb_endp3type , 8 , 0
Data _usb_endp3interval
#endif

#if _usb_hids > 0
_usb_hid_reportdescriptor:

Data 65                                                     ' Length including size is 45 bytes
Data &H05 , &H01                                            ' USAGE_PAGE (Generic Desktop)
Data &H09 , &H05                                            ' USAGE (Gamepad)
Data &HA1 , &H01                                            ' COLLECTION (Application)
'7 bytes

Data &H09 , &H01                                            '   USAGE (Pointer)
Data &HA1 , &H00                                            '   COLLECTION (Physical)
Data &H09 , &H33                                            '     USAGE (Rx)
Data &H09 , &H34                                            '     USAGE (Ry)
Data &H09 , &H35                                            '     USAGE (Rz)
Data &H15 , &H00                                            '     LOGICAL_MINIMUM (0)
Data &H25 , &H1E                                            '     LOGICAL_MAXIMUM (30)
Data &H95 , &H03                                            '     REPORT_COUNT (3)
Data &H75 , &H05                                            '     REPORT_SIZE (5)
Data &H81 , &H02                                            '     INPUT (Data,Var,Abs)
Data &H95 , &H01                                            '     REPORT_COUNT (1)
Data &H75 , &H01                                            '     REPORT_SIZE (1)
Data &H81 , &H03                                            '     INPUT (Cnst,Var,Abs)
Data &H09 , &H30                                            '     USAGE (X)
Data &H09 , &H31                                            '     USAGE (Y)
Data &H15 , &HFF                                            '     LOGICAL_MINIMUM (-1)
Data &H25 , &H01                                            '     LOGICAL_MAXIMUM (1)
Data &H95 , &H02                                            '     REPORT_COUNT (2)
Data &H75 , &H02                                            '     REPORT_SIZE (2)
Data &H81 , &H02                                            '     INPUT (Data,Var,Abs)
Data &HC0                                                   '   END_COLLECTION
'41 bytes

Data &H05 , &H09                                            '   USAGE_PAGE (Button)
Data &H19 , &H01                                            '   USAGE_MINIMUM (Button 1)
Data &H29 , &H0C                                            '   USAGE_MAXIMUM (Button 12)
Data &H15 , &H00                                            '   LOGICAL_MINIMUM (0)
Data &H25 , &H01                                            '   LOGICAL_MAXIMUM (1)
Data &H95 , &H0C                                            '   REPORT_COUNT (12)
Data &H75 , &H01                                            '   REPORT_SIZE (1)
Data &H81 , &H02                                            '   INPUT (Data,Var,Abs)
Data &HC0                                                   ' END_COLLECTION
'17 bytes

#endif

'*****************************String descriptors********************************
'Yes, they MUST be written like "t","e","s","t".  Doing so pads them with
'0's.  If you write it like "test," I promise you it won't work.

'Default language descriptor (index 0)
_usb_langdescriptor:
Data 4 , 4 , _usb_desc_string , 09 , 04                     '&h0409 = English

'Manufacturer Descriptor (unicode)
_usb_mandescriptor:
Data 14 , 14 , _usb_desc_string
Data "o" , "l" , "l" , "o" , "p" , "a"

'Product Descriptor (unicode)
_usb_proddescriptor:
Data 44 , 44 , _usb_desc_string
Data "o" , "l" , "l" , "o" , "p" , "a" , "'" , "s" , " " , "g" , "a" , "m" , "e" , "p" , "a" , "d" , " "
Data "v" , "1" , "." , "0"



'*******************************************************************************



'*******************************************************************************
'******************************** Subroutines **********************************
'*******************************************************************************

Sub Usb_processsetup(txstate As Byte)
Senddescriptor = 0

   'Control transfers reset the sync bits like so
   Txstate = _usb_setup_sync

   'These are the standard device, interface, and endpoint requests that the
   'USB spec requires that we support.
   Select Case _usb_rx_buffer(2)
      'Standard Device Requests
      Case &B10000000:
         Select Case _usb_rx_buffer(3)
'            CASE _usb_REQ_GET_STATUS:
            Case _usb_req_get_descriptor:
               Select Case _usb_rx_buffer(5)
                  Case _usb_desc_device:
                     'Send the device descriptor
                     #if _usb_use_eeprom = 1
                        Readeeprom _usb_eepromaddrl , _usb_devicedescriptor
                     #else
                        Restore _usb_devicedescriptor
                     #endif
                     Senddescriptor = 1
                  Case _usb_desc_config:
                     'Send the configuration descriptor
                     #if _usb_use_eeprom = 1
                        Readeeprom _usb_eepromaddrl , _usb_configdescriptor
                     #else
                        Restore _usb_configdescriptor
                     #endif
                     Senddescriptor = 1
                  Case _usb_desc_string:
                     Select Case _usb_rx_buffer(4)
                        Case 0:
                           'Send the language descriptor
                           #if _usb_use_eeprom = 1
                              Readeeprom _usb_eepromaddrl , _usb_langdescriptor
                           #else
                              Restore _usb_langdescriptor
                           #endif
                           Senddescriptor = 1
                        Case 1:
                           'Send the manufacturer descriptor
                           #if _usb_use_eeprom = 1
                              Readeeprom _usb_eepromaddrl , _usb_mandescriptor
                           #else
                              Restore _usb_mandescriptor
                           #endif
                           Senddescriptor = 1
                        Case 2:
                           'Send the product descriptor
                           #if _usb_use_eeprom = 1
                              Readeeprom _usb_eepromaddrl , _usb_proddescriptor
                           #else
                              Restore _usb_proddescriptor
                           #endif
                           Senddescriptor = 1
                     End Select
               End Select
'            CASE _usb_REQ_GET_CONFIG:
         End Select
      Case &B00000000:
         Select Case _usb_rx_buffer(3)
'            CASE _usb_REQ_CLEAR_FEATURE:
'            CASE _usb_REQ_SET_FEATURE:
            Case _usb_req_set_address:
               'USB status reporting for control writes
               Call Usb_send(txstate , 0)
               While Txstate._usb_txc = 0 : Wend
               'We are now addressed.
               _usb_deviceid = _usb_rx_buffer(4)
'            CASE _usb_REQ_SET_DESCRIPTOR:
            Case _usb_req_set_config:
               'Have to do status reporting
               Call Usb_send(txstate , 0)
         End Select
      'Standard Interface Requests
      Case &B10000001:
         Select Case _usb_rx_buffer(3)
'            CASE _usb_REQ_GET_STATUS:
'            CASE _usb_REQ_GET_IFACE:
            Case _usb_req_get_descriptor
            '_usb_rx_buffer(4) is the descriptor index and (5) is the type
               Select Case _usb_rx_buffer(5)
                  Case _usb_desc_report:
                     #if _usb_use_eeprom = 1
                        Readeeprom _usb_eepromaddrl , _usb_hid_reportdescriptor
                     #else
                        Restore _usb_hid_reportdescriptor
                     #endif
                     Senddescriptor = 1
'                  CASE _usb_DESC_PHYSICAL

'                  CASE _USB_DESC_HID

               End Select
         End Select
      Case &B00100001:
         'Class specific SET requests
         Select Case _usb_rx_buffer(3)
            'CASE _usb_REQ_SET_REPORT:
            Case _usb_req_set_idle:
               Idlemode = 1
               'Do status reporting
               Call Usb_send(txstate , 0)
            'CASE _usb_REQ_SET_PROTOCOL:
         End Select
   End Select

   If Senddescriptor = 1 Then
      Call Usb_senddescriptor(txstate , _usb_rx_buffer(8))
   End If

End Sub

Sub Usb_senddescriptor(txstate As Byte , Maxlen As Byte)
   'Break the descriptor into packets and send to TxState
   Local Size As Byte
   Local I As Byte
   Local J As Byte
   Local Timeout As Word

   #if _usb_use_eeprom = 1
      'EEPROM access is a little funky.  The size has already been fetched
      'and stored in _usb_EEPROMADDRL, and the address of the descriptor
      'is now in the EEAR register pair.

      Size = _usb_eepromaddrl

      'Fetch the location of the descriptor and use it as an address pointer
      push R24
      in R24, EEARL
      sts {_USB_EEPROMADDRL}, R24
      in R24, eearH
      sts {_USB_EEPROMADDRH}, R24
      pop R24

   #else
      Read Size
   #endif



   If Maxlen < Size Then Size = Maxlen

   I = 2
   For J = 1 To Size
      Incr I
      #if _usb_use_eeprom = 1
         Incr _usb_eepromaddr
         Readeeprom Txstate(i) , _usb_eepromaddr
      #else
         Read Txstate(i)
      #endif


      If I = 10 Or J = Size Then
         I = I - 2
         Call Usb_send(txstate , I)
         While Txstate._usb_txc = 0
            Timeout = 0
            'To prevent an infinite loop, check for reset here
            While _usb_pin._usb_dminus = 0
               Incr Timeout
               If Timeout = 1000 Then                       '
                  Call Usb_reset()
                  Exit Sub
               End If
            Wend
         Wend
         I = 2
      End If
   Next
End Sub

Sub Usb_send(txstate As Byte , Byval Count As Byte)

   'Calculates and adds the CRC16,adds the DATAx PID,
   'and signals to the ISR that the data is ready to be sent.
   '
   '"Count" is the DATA payload size.  Range is 0 to 8. Do not exceed 8!

   'Reset all the flags except TxSync and RxSync
   Txstate = Txstate And _usb_syncmask

   'Calculate the 16-bit CRC
   _usb_crc = 0
   If Count <> 0 Then
      _usb_crc = Crc16uni(txstate(3) , Count , &HFFFF , &H8005 , 1 , 1)
      Toggle _usb_crc
   End If

   'Bytes to transmit will be PID + DATA payload + CRC16
   Count = Count + 3
   Txstate = Txstate + Count

   Txstate(count) = Low(_usb_crc)
   Incr Count
   Txstate(count) = High(_usb_crc)


   'Add the appropriate DATAx PID
   Txstate(2) = _usb_pid_data1
   If Txstate._usb_txsync = 0 Then
      Txstate(2) = _usb_pid_data0
   End If

   'The last step is to signal that the packet is Ready To Transmit
   Txstate._usb_rtt = 1
   Txstate._usb_txc = 0
End Sub

Sub Usb_reset()
   'Reset the receive flags
   _usb_status._usb_rtr = 1
   _usb_status._usb_rxc = 0

   'Reset the transmit flags
   _usb_tx_status = _usb_endp_init
   #if Varexist( "_usb_Endp2Addr")
   _usb_tx_status2 = _usb_endp_init
   #endif
   #if Varexist( "_usb_Endp3Addr")
   _usb_tx_status3 = _usb_endp_init
   #endif

   'Reset the device ID to 0
   _usb_deviceid = 0

   Idlemode = 0
End Sub





'Display character on LED matrix
Sub Push_char(byval Char As Byte)
   Dim Space_position As Integer
   Space_position = 7
   Dim Data_position As Byte
   Dim Animation_frame As Byte
   For Animation_frame = 0 To 8
      For B = 0 To 7
         Disp_addr = B + 1
         If B > Space_position Then
            'New char
            Data_position = B - Space_position
            Decr Data_position
            Select Case Char
               Case F : Disp_data = Lookup(data_position , Fchar)
               Case X : Disp_data = Lookup(data_position , Xchar)
               Case Y : Disp_data = Lookup(data_position , Ychar)
               Case Z : Disp_data = Lookup(data_position , Zchar)
               Case Lock : Disp_data = Lookup(data_position , Lockchar)
            End Select
         Elseif B = Space_position Then
            'Space
            Disp_data = 0
         Else
            'Old char
            Data_position = B + Animation_frame
            Select Case Current_axis
               Case F : Disp_data = Lookup(data_position , Fchar)
               Case X : Disp_data = Lookup(data_position , Xchar)
               Case Y : Disp_data = Lookup(data_position , Ychar)
               Case Z : Disp_data = Lookup(data_position , Zchar)
               Case Lock : Disp_data = Lookup(data_position , Lockchar)
            End Select
         End If
         Gosub Disp_write
      Next B

      Decr Space_position
      Waitms 30
   Next Animation_frame
   Current_axis = Char
End Sub


'Display character on LED matrix
Sub Display_char(byval Char As Byte)
   For B = 0 To 7
      Disp_addr = B + 1
      Select Case Char
        Case F : Disp_data = Lookup(b , Fchar)
        Case X : Disp_data = Lookup(b , Xchar)
        Case Y : Disp_data = Lookup(b , Ychar)
        Case Z : Disp_data = Lookup(b , Zchar)
        Case Lock : Disp_data = Lookup(b , Lockchar)
      End Select
      Gosub Disp_write
   Next B
End Sub


Function Get_rotary() As Byte
   If Rotary1 = 0 Then
      Get_rotary = 1
   Elseif Rotary2 = 0 Then
      Get_rotary = 2
   Elseif Rotary3 = 0 Then
      Get_rotary = 3
   Else
      Get_rotary = 0
   End If
End Function


'Set LED matrix brightness (Minimum &H00, Maximum &H0F)
Sub Set_intensity(byval Level As Byte)
   If Level > &H0F Then Level = &H0F
   Disp_addr = &H0A
   Disp_data = Level
   Gosub Disp_write
End Sub


'Initialize MAX7221
Max7221_init:
   Disp_addr = &H0C : Disp_data = 0                         ' Shutdown On
   Gosub Disp_write

   Disp_addr = &H09 : Disp_data = &H00                      ' No Decode
   Gosub Disp_write

   Set_intensity &H07                                       ' Brightness 50%

   Disp_addr = &H0B : Disp_data = 7                         ' Digits/Columns 8
   Gosub Disp_write

   Disp_addr = &H0F : Disp_data = 0                         ' Test Off
   Gosub Disp_write

   Disp_addr = &H0C : Disp_data = 1                         ' Shutdown Off
   Gosub Disp_write
Return


'Turns all LED's off
Clear_display:
   For B = 1 To 8
      Disp_addr = B
      Disp_data = &H00
      Gosub Disp_write
   Next B
Return


'Write data to the MAX7221
Disp_write:
   Disable Interrupts
   Reset Disp_din
   Reset Disp_clk
   Reset Disp_load
   Shiftout Disp_din , Disp_clk , Disp_addr , 1
   Shiftout Disp_din , Disp_clk , Disp_data , 1
   Set Disp_load
   Enable Interrupts
Return

Handle_joystick_rotation:
   'Get current rotary joystick position
   Rotary_current = Get_rotary()

   'Capture joystick rotation
   If Rotary_current <> Rotary_old And Rotary_current <> 0 Then
      Rotary_dif = Rotary_current - Rotary_old
      If Rotary_dif = 1 Or Rotary_dif = -2 Then
         'Clockwise rotation
         Incr Rotary_stack
      Else
         'Counter Clockwise rotation
         Decr Rotary_stack
      End If
      'Set last rotary joystick position to current position
      Rotary_old = Rotary_current
   End If

     'Handle rotation stack
    If Rotary_status = Rotate_ready Then
      If Rotary_stack < 0 Then
         'Set flag for Counter Clockwise rotation
         Rotary_status = Rotating_ccw
         'Begin key down clock
         Rotary_clock = Key_down_cycles
         'Adjust p1 stack
         Incr Rotary_stack
      Elseif Rotary_stack > 0 Then
         'Set flag for Clockwise rotation
         Rotary_status = Rotating_cw
         'Begin key down clock
         Rotary_clock = Key_down_cycles
         'Adjust P1 stack
         Decr Rotary_stack
      End If
    Elseif Rotary_clock > 0 Then
      'Increment rotary cycle clock
      Decr Rotary_clock
      If Rotary_clock <= 0 Then
         If Rotary_status = Rotating_ccw Or Rotary_status = Rotating_cw Then
            'Begin wait between key presses
             Rotary_clock = Key_wait_cycles
            'Release rotational flag
            Rotary_status = Rotate_waiting
         Else
            'Finalize rotation event
            Rotary_status = Rotate_ready
         End If
      End If
    End If
Return



'Check feed rate pot
Poll_feedrate:
   W = Getadc(0)

   'Set slider level (lower 10 bit ADC to 4 bit resolution)
   Shift W , Right , 6
   Spinner_level = W

   'Set 7 segment display (lower 4 bit slider to 3 BCD resolution)
   Shift W , Right , 1
   If Bcd_level <> W Then
      Bcd_level = W
      Bcd0 = Bcd_level.0
      Bcd1 = Bcd_level.1
      Bcd2 = Bcd_level.2
   End If
Return



'Characters
Fchar:
Data &B11111111 , &B11111111 , &B00011011 , &B00011011 , &B00011011 , &B00000011 , &B00000011 , &B00000000

Xchar:
Data &B01000001 , &B01100011 , &B00110110 , &B00011100 , &B00011100 , &B00110110 , &B01100011 , &B01000001

Ychar:
Data &B00000001 , &B00000011 , &B00000111 , &B01111100 , &B01111100 , &B00000111 , &B00000011 , &B00000001

Zchar:
Data &B01000011 , &B01100011 , &B01110011 , &B01111011 , &B01101111 , &B01100111 , &B01100011 , &B01100001

Lockchar:
Data &B01111000 , &B01111110 , &B01111001 , &B01001001 , &B01111001 , &B01111110 , &B01111000 , &B00000000

'Axis Spinner to 5 bit resolution Mach3 Optimization
Axis_negative:
Data 15 , 15 , 09 , 09 , 08 , 08 , 07 , 07 , 06 , 05 , 04 , 03 , 02 , 01 , 00 , 00
Axis_positive:
Data 15 , 15 , 21 , 21 , 22 , 22 , 23 , 23 , 24 , 25 , 26 , 27 , 28 , 29 , 30 , 30